﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Malovani
{
    public partial class Window : Form
    {
        private Color initColor = Color.Yellow;
        private Color previewColor = Color.RoyalBlue;

        private Pen pen = new Pen (Color.Orange);
        private Brush brush = new SolidBrush (Color.Gray);

        private int X0, Y0;
        private bool stisk = false;
        private Bitmap save;

        private void paint (MouseEventArgs e, Pen p, Brush b)
        {
            Graphics g = Graphics.FromImage(pictureBox.Image);
            g.DrawImage(save, 0, 0);

            if (e.Button == MouseButtons.Right)
            {
                int X1 = e.X;
                int Y1 = e.Y;

                int X = Math.Min(X0, X1);
                int Y = Math.Min(Y0, Y1);
                int W = Math.Abs(X0 - X1);
                int H = Math.Abs(Y0 - Y1);

                // if (b != null)
                g.FillRectangle(b, X, Y, W, H);
                g.DrawRectangle(p, X, Y, W, H);
            }
            else if (e.Button == MouseButtons.Middle)
            {
                // if (b != null)
                g.FillEllipse(b, X0, Y0, e.X - X0, e.Y - Y0);
                g.DrawEllipse(p, X0, Y0, e.X - X0, e.Y - Y0);
            }
            else
            {
                g.DrawLine(p, X0, Y0, e.X, e.Y);
            }

            pictureBox.Invalidate();
        }
        
        public Window()
        {
            InitializeComponent();
            pictureBox.Image = new Bitmap
                (pictureBox.Width, pictureBox.Height);

            Graphics g = Graphics.FromImage (pictureBox.Image);
            Brush b = new SolidBrush (initColor);
            g.FillRectangle (b, 0, 0, pictureBox.Width, pictureBox.Height);
        }

        private void testMenu_Click(object sender, EventArgs e)
        {
            TreeNode item = new TreeNode ();
            item.Text = "new item";
            tree.Nodes.Add (item);

            TreeNode item2 = new TreeNode();
            item2.Text = "sub-item";
            item.Nodes.Add(item2);

            item.Expand ();

            /*
            Graphics g = Graphics.FromImage(pictureBox.Image);
            Pen p  = new Pen (Color.Red);
            p.Width = 3;
            g.DrawLine (p, 10, 10, 100, 100);


            for (int i = 0; i < 8; i++)
                for (int j = 0; j < 8; j++)
                {
                    const int K = 20;
                    g.DrawRectangle (pen, K*i, K*j, K-2, K-2);
                }

            pictureBox.Invalidate ();
            */
        }

        private void quitMenu_Click(object sender, EventArgs e)
        {
            Close ();
        }

        private void pictureBox_MouseDown(object sender, 
                                          MouseEventArgs e)
        {
            X0 = e.X;
            Y0 = e.Y;
            stisk = true;
            save = new Bitmap (pictureBox.Image);
        }

        private void pictureBox_MouseMove(object sender, MouseEventArgs e)
        {
            if (stisk)
            {
                Pen p = new Pen (previewColor);
                Brush b = new SolidBrush (Color.FromArgb(0,0,0,0));
                paint (e, p, b);
            }
        }

        private void pictureBox_MouseUp(object sender, MouseEventArgs e)
        {
            if (stisk)
            {
               paint (e, pen, brush);
               stisk = false;
               save = null;
            }
        }

        private void pictureBox_Resize(object sender, EventArgs e)
        {
        }

        private static int max(int a, int b)
        {
            return a > b ? a : b;
        }

        private void pictureBox_SizeChanged(object sender, EventArgs e)
        {
            Size s1 = pictureBox.Size;
            Size s2 = pictureBox.Image.Size;
            int W = max(s1.Width, s2.Width);
            int H = max(s1.Height, s2.Height);
            Bitmap b = new Bitmap(W, H);

            Graphics g = Graphics.FromImage(b);
            Brush br = new SolidBrush(initColor);
            g.FillRectangle(br, 0, 0, pictureBox.Width, pictureBox.Height);
            g.DrawImage(pictureBox.Image, 0, 0);
            pictureBox.Image = b;
        }

        private void colorPanel1_MouseDown(object sender, MouseEventArgs e)
        {
            Panel panel = (sender as Panel);
            Color color = panel.BackColor;

            if (e.Button == MouseButtons.Left)
                pen = new Pen(color);
            else if (e.Button == MouseButtons.Right)
                brush = new SolidBrush(color);
            else if (e.Button == MouseButtons.Middle)
            {
                // DialogResult result = colorDialog.ShowDialog();
                // if (result == DialogResult.OK)
                if (colorDialog.ShowDialog () == DialogResult.OK)
                {
                    panel.BackColor = colorDialog.Color;
                    pen = new Pen (colorDialog.Color);
                }
            }

        }

        private int cnt = 3;

        private void addPanelMenu_Click(object sender, EventArgs e)
        {
            cnt ++;
            int dist = colorPanel2.Left - colorPanel1.Left - colorPanel1.Width;
            Panel p = new Panel ();
            p.Width = colorPanel1.Width;
            p.Height = colorPanel1.Height;
            p.Top = colorPanel1.Top;
            p.Left = colorPanel1.Left + (cnt-1) * (colorPanel1.Width + dist);
            p.BackColor = Color.Green;
            // toolPanel.Controls.Add (p);
            p.Parent = toolPanel;
            p.MouseDown += colorPanel1_MouseDown;

            // MyArea a = new MyArea ();
            // a.Parent = pictureBox;
        }

        private void addAreaMenu_Click(object sender, EventArgs e)
        {
            MyArea a = new MyArea ();
            addArea (null, a);
        }

        /* -------------------------------------------------------------- */

        private void saveMenu_Click(object sender, EventArgs e)
        {
            if (saveDialog.ShowDialog() == DialogResult.OK)
            {
                /*
                Rectangle r = pictureBox.DisplayRectangle;
                Point loc = pictureBox.Location;
                Size s = new Size (r.Width, r.Height);

                Bitmap b = new Bitmap(s.Width, s.Height);
                Graphics g = Graphics.FromImage(b);

                g.CopyFromScreen(loc.X, loc.Y, 0, 0, s);
                */

                Bitmap b = new Bitmap(pictureBox.Width, pictureBox.Height);
                Rectangle r = pictureBox.DisplayRectangle;
                pictureBox.DrawToBitmap(b, r);

                b.Save(saveDialog.FileName); 
            }

        }

        private void eraseMenu_Click(object sender, EventArgs e)
        {
            List<Control> list = new List<Control> ();
            foreach (Control c in pictureBox.Controls)
                list.Add (c);
            foreach (Control c in list)
                c.Parent = null;

            foreach (Control c in pictureBox.Controls)
                c.Parent = null;
        }

        /* -------------------------------------------------------------- */

        // MyArea[] top = { };
        /*
        MyArea [] n = new MyArea [top.Length+1];
        for (int i = 0; i < top.Length; i ++)
            n[i] = top[i];
        n[n.Length-1] = newArea;
        top = n;
        */

        private List<MyArea> top = new List<MyArea>();

        public void addArea (MyArea aboveArea, MyArea newArea)
        {
            MyInfo info = new MyInfo();
            info.area = newArea;

            TreeNode node = new TreeNode ();
            node.Text = info.ToString ();
            node.Tag = info;

            newArea.info = info;
            newArea.node = node;
            newArea.win = this;

            if (aboveArea == null)
            {
                top.Add (newArea);
                tree.Nodes.Add (node);
                newArea.Parent = pictureBox;
            }
            else
            {
                aboveArea.below.Add (newArea);
                aboveArea.node.Nodes.Add (node);
                newArea.Parent = aboveArea;
            }
        }

        public void selectArea (MyArea a)
        {
            tree.SelectedNode = a.node;
        }
        private void tree_AfterSelect(object sender, TreeViewEventArgs e)
        {
            MyArea a = null;
            TreeNode n = tree.SelectedNode;
            if (n != null)
            {
                MyInfo t = n.Tag as MyInfo;
                if (t != null) 
                    a = t.area;
            }

            grid.SelectedObject = a;
            // grid.SelectedObject = (a != null) ? a.info : null;
        }

    }
}
